part of '../vrchat_dart.dart';

/// Intercept auth cookies to use for websocket streaming
class AuthCookieInterceptor extends dio.Interceptor {
  /// Intercept auth cookies from requests
  @override
  void onRequest(
    dio.RequestOptions options,
    dio.RequestInterceptorHandler handler,
  ) {
    _parseAuthCookie(options.headers['cookie']?.split('; '));
    handler.next(options);
  }

  /// Intercept auth cookies from responses
  @override
  void onResponse(
    dio.Response response,
    dio.ResponseInterceptorHandler handler,
  ) {
    _parseAuthCookie(response.headers['set-cookie']);
    handler.next(response);
  }

  String? _parseAuthCookie(List<String>? headers) {
    final authToken = headers
        // 'auth=authcookie_*****; Max-Age=604800; Path=/; Expires=Mon, 26 Jul 2021 20:25:36 GMT; HttpOnly'
        ?.firstWhereOrNull((cookie) => cookie.contains('auth=authcookie_'))
        // [ 'auth=authcookie_*****', 'Max-Age=604800', 'Path=/', 'Expires=Mon, 26 Jul 2021 20:25:36 GMT', 'HttpOnly' ]
        ?.split(';')
        // 'auth=authcookie_*****'
        .firstOrNull
        // 'authcookie_*****'
        ?.replaceAll('auth=', '');

    if (authToken != null) {
      VrcStreaming._authToken = authToken;
    }
  }
}
